//SimpleCE - port of Simple to PocketPC 
//(c) Piotr Gawrysiak 2002 - pgawrysiak@supermedia.pl
//http://bolek.ii.pw.edu.pl/~gawrysia
//License - GPL

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

#include <windows.h>
#include "resource.h"
#include "gx.h"

//Global variables
//Global suspend trigger
int GlobalStop;

//Application instance handle
HINSTANCE hInst;	

//GAPI variables
GXDisplayProperties gx_displayprop;
GXKeyList gx_keylist;
unsigned short BackBuffer[320*240];			
unsigned short PlayField[320*240];
unsigned short * pFramesBitmap; //Frames bitmap
unsigned short * pStatusBitmap; //Status bitmap
unsigned short * pSplashBitmap; //Splash bitmap

unsigned short * memstartBackBuffer;
unsigned short * memstartPlayField;  //first pixel addresses 


int FramesWidth, FramesHeight, StatusWidth, StatusHeight; //Loaded bitmap sizes - nazwac to jakos po ludzku!!!
int SplashWidth, SplashHeight;

//Balls positions and velocities array
struct { short x, y; short vx, vy; short type; short frame; } BallInfo[100];

//Lines array for collision detection
//This is 240x320 divided into 2x2 squares 
int lines[120][160];

//Ship position and direction
//Ship position is given in lines array coordinates plus displacement
DWORD sx=4;
DWORD sy=80;
int svx = 0; //This applies to displacement
int svy = 0;
int nvx = 0; //New direction 
int nvy = 0;
int SIcon = 0;
BOOL LeaveTrail=FALSE; //Are we painting new trail?
BOOL InGame=FALSE; //Are we displaying splash screen or in game?
BOOL GameStart=TRUE; //Are we displaying intro screen?
BOOL IsGameOver=FALSE; //Are we displaying game over screen?

//Game level
//This affects number of balls displayed
int GameLevel=1;
int Ships=3;
int Fill=0;
int LevelTime=90;

//Game delay
DWORD	GameDelay = 10; //Global game delay - applies to balls and to ship
DWORD	LastTime = 0;

//Fill amount
int RNFill;
int LNFill;


// Forward declarations
BOOL InitInstance (HINSTANCE, int);
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);
unsigned short * ConvertBitmap(LPTSTR BitmapName, int * Width, int * Height, int SX, int SY, int HX, int HY);
unsigned short CalcColor(int r, int g, int b);
void ClearScreen(unsigned short Color);
void Flip();
void UpdateFrame();
int  Rnd(int limit);
void CalcPos(void);
void FillAreas(int ax, int ay,int bx, int by);
void BlendLines(void);
void LifeOut(void);
void ResetLevel(void);
void NextLevel(void);
void GameOver(void);
BOOL DrawLines(void);
void DrawStatus(void);
void DrawSplash(int Type);
void DrawTime(void);
void ClearPlayField(void);
void UpdateScreen(void);
BOOL DrawBalls(void);
void ReadInput(void);
BOOL ReadInput(WPARAM wParam);
void BallInit();
void LinesInit();

//Function definitions
int WINAPI WinMain(	HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	MSG msg;

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	//Start game timer

	while( 1 )
    {

        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )// Are there any messages?
        {
            if( !GetMessage( &msg, NULL, 0, 0 ) )
            {
                return msg.wParam;
            }
                TranslateMessage(&msg);
                DispatchMessage(&msg);
            
        }
        else
        {
			if (!GlobalStop)
			{
				if (InGame) UpdateFrame(); else Flip(); //As frequently as possible
			}
        }        
    }

	return msg.wParam;
}// of WinMain

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd = NULL;
	WNDCLASS wc;

	hInst = hInstance;		// Store instance handle in our global variable
	
	//If it is already running, then focus on the window
	hWnd = FindWindow(L"SimpleCE", L"SimpleCE");	
	if (hWnd) 
	{
		SetForegroundWindow ((HWND) (((DWORD)hWnd) | 0x01));    
		return 0;
	} 

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = NULL;
    wc.hCursor = NULL;
    wc.hbrBackground = (HBRUSH )GetStockObject(WHITE_BRUSH);
    wc.lpszClassName = L"SimpleCE";
	wc.lpszMenuName = NULL;
    RegisterClass(&wc);
	
	// Create a window
    hWnd = CreateWindowEx(WS_EX_TOPMOST,
                          L"SimpleCE",
                          L"SimpleCE",
                          WS_POPUP,
                          0,
                          0,
                          GetSystemMetrics(SM_CXSCREEN),
                          GetSystemMetrics(SM_CYSCREEN),
                          NULL,
                          NULL,
                          hInstance,
                          NULL);
	if (!hWnd)
	{	
		return FALSE;
	}


	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);
	SetFocus(hWnd);

	// Attempt to take over the screen
	if (GXOpenDisplay( hWnd, GX_FULLSCREEN) == 0)
		return FALSE;

	// Get display properties
	gx_displayprop = GXGetDisplayProperties();

	// Check for 16 bit color display
	if (gx_displayprop.cBPP != 16)
	{
		// Only dealing with 16 bit color in this code
		GXCloseDisplay();
		MessageBox(hWnd,L"Only supporting 16bit color display",L"Error!", MB_OK);
		return FALSE;
	}

	//Set buffer pointers
	if (gx_displayprop.cbyPitch<0)
	{
		memstartBackBuffer = (unsigned short *)BackBuffer + ((gx_displayprop.cyHeight-1)*-1*(gx_displayprop.cbyPitch>>1));
		memstartPlayField = (unsigned short *)PlayField + ((gx_displayprop.cyHeight-1)*-1*(gx_displayprop.cbyPitch>>1));
	} 
	else
	{
		memstartBackBuffer = (unsigned short *)BackBuffer;
		memstartPlayField = (unsigned short *)PlayField;
	}

	// Take over button handling
	GXOpenInput();

	// Get default buttons for up/down etc.
	gx_keylist = GXGetDefaultKeys(GX_NORMALKEYS);

	//Load frames bitmap
	pFramesBitmap = ConvertBitmap(MAKEINTRESOURCE(IDB_FRAMES),&FramesWidth,&FramesHeight,0,0,320,32);

	//Load status bitmap
	pStatusBitmap = ConvertBitmap(MAKEINTRESOURCE(IDB_STATUS),&StatusWidth,&StatusHeight,0,0,240,14);

	//Load splash bitmap
	pSplashBitmap = ConvertBitmap(MAKEINTRESOURCE(IDB_SPLASH),&SplashWidth,&SplashHeight,0,0,200,200);

	//Initialize BallInfo (przerobic)
	BallInit();

	//Initialize lines array (frame around with space on top for score line)
	LinesInit();


/*	CalcPos();
	DrawStatus();
	DrawLines(); 
*/ 

//	DrawStatus();
	DrawSplash(1);

	GlobalStop = 0;

	return TRUE;
}

void CALLBACK TimerProc(HWND hWnd, UINT uMsg, UINT idEvent,DWORD dwTime)
{
	if (InGame)
	{
			LevelTime=LevelTime-1;
			DrawTime();
			if (LevelTime<1) LifeOut();
	}
			return;
}//of TimerProc


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	switch (message) 
	{
		
		case WM_CREATE:
			SetTimer(hWnd, 1, 1000, (TIMERPROC)TimerProc);
				
			break;

		case WM_LBUTTONDOWN:
             //Koczymy dziaanie programu po pukniciu rysikiem
			if (InGame) PostMessage(hWnd, WM_CLOSE, 0, 0); 
			else 
			{
				if (IsGameOver==TRUE)
				{
					BallInit();
					LinesInit();
					IsGameOver=FALSE;
					GameStart=TRUE;
					DrawSplash(1);
				} 
				else
				if (GameStart==TRUE)
				{//Start game
					DrawSplash(3);
					GameStart=FALSE;
				} else 
				{ 
						InGame=TRUE;
						ResetLevel();
				}
			}
			 break;

		case WM_KEYDOWN:

			if (InGame) {
				if (ReadInput(wParam) == FALSE)
				{
					//SendMessage(hWnd,WM_CLOSE,0,0);
				}
			}
			else
			{
				if (IsGameOver==TRUE)
				{
					BallInit();
					LinesInit();
					IsGameOver=FALSE;
					GameStart=TRUE;
					DrawSplash(1);
				} 
				else
				if (GameStart==TRUE)
				{//Start game
					DrawSplash(3);
					GameStart=FALSE;
				} else 
				{ 
						InGame=TRUE;
						ResetLevel();
				}
			}
			break;
		
		case WM_DESTROY:

			KillTimer(hWnd,1);

			GXCloseInput();
			GXCloseDisplay();

			PostQuitMessage(0);
			break;

		case WM_KILLFOCUS:
			GlobalStop = 1;
			GXSuspend();
			break;

		case WM_SETFOCUS:
			GlobalStop = 0;
			GXResume();
			break;

		/*case WM_PAINT:
			RECT	rect;
			GetWindowRect(hWnd, &rect);
			ValidateRect(hWnd, &rect);
			break;*/

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}// of WndProc


//------------------------------------------------------------------
//Graphic functions

unsigned short * ConvertBitmap(LPTSTR BitmapName, int * Width, int * Height, int SX, int SY, int HX, int HY)
{

HBITMAP SourceBitmap, TargetBitmap;
BITMAP bm; // bitmap structure to get info about our bitmap

unsigned char *pFramesBitmap, *pSrc;
unsigned short *pTemp;
register unsigned short *pDest, usPixel;
int pitch, x, y, destpitch;

	SourceBitmap = LoadBitmap(hInst, BitmapName);  

	GetObject(SourceBitmap, sizeof(BITMAP), &bm);

	void *pBuffer;

	BITMAPINFO  dibInfo;
	dibInfo.bmiHeader.biBitCount = 24;
	dibInfo.bmiHeader.biClrImportant = 0;
	dibInfo.bmiHeader.biClrUsed = 0;
	dibInfo.bmiHeader.biCompression = 0;
	dibInfo.bmiHeader.biHeight = bm.bmHeight;
	dibInfo.bmiHeader.biPlanes = 1;
	dibInfo.bmiHeader.biSize = 40;
	dibInfo.bmiHeader.biSizeImage = bm.bmWidth*bm.bmHeight*3;
	dibInfo.bmiHeader.biWidth = bm.bmWidth;
	dibInfo.bmiHeader.biXPelsPerMeter = 3780;
	dibInfo.bmiHeader.biYPelsPerMeter = 3780;
	dibInfo.bmiColors[0].rgbBlue = 0;
	dibInfo.bmiColors[0].rgbGreen = 0;
	dibInfo.bmiColors[0].rgbRed = 0;
	dibInfo.bmiColors[0].rgbReserved = 0;

	HDC hDC = GetDC(NULL);
	TargetBitmap = CreateDIBSection(
		hDC,
		&dibInfo,
		DIB_RGB_COLORS,
		(void**)&pBuffer,
		NULL,
		0);

	ReleaseDC(NULL, hDC);

	HDC memDc;
	memDc=CreateCompatibleDC(NULL);

	HDC targetDc;
	targetDc=CreateCompatibleDC(NULL);
	
	HBITMAP hOldBitmap1 = (HBITMAP) SelectObject(memDc, SourceBitmap);
	HBITMAP hOldBitmap2 = (HBITMAP) SelectObject(targetDc, TargetBitmap);

	BitBlt(targetDc, 0, 0, HX, HY, memDc, SX, SY, SRCCOPY);

	pFramesBitmap = (unsigned char *) pBuffer;
	pitch = ((bm.bmWidth*3)+3) & 0xfffc; // bytes per line, dword-aligned
    destpitch = (bm.bmWidth+1) & 0xfffe;

	// Allocate a temp buffer to convert the color format and flip it upside-down
    pTemp = (unsigned short *)VirtualAlloc(NULL, pitch*bm.bmHeight, MEM_COMMIT, PAGE_READWRITE);

	// Convert the color format from RGB (24-bit) to RGB565 and flip it vertically
    for (y=0; y<bm.bmHeight; y++)
    {
      pDest = pTemp + y * destpitch;
      pSrc = (unsigned char *)pFramesBitmap + (bm.bmHeight-y-1)*pitch;
      for (x=0; x<bm.bmWidth; x++)
         {
         usPixel = pSrc[0] >> 3; // blue
         usPixel |= (pSrc[1] & 0xfc) << 3; // green
         usPixel |= (pSrc[2] & 0xf8) << 8; // red
         *pDest = usPixel;
         pDest++;
         pSrc += 3;
         }
      }

	DeleteObject(SourceBitmap);
	DeleteObject(TargetBitmap);

	*Width = bm.bmWidth;
	*Height = bm.bmHeight;
	return pTemp;

}//of ConvertBitmap

//
//iCornerX, iCornerY
//ax, ay
//dx, dy
//cx, cy
void GxBitbltCut(unsigned short *pSrc, int iCornerX, int iCornerY, int ax, int ay, int dx, int dy, int cx, int cy, unsigned short * pDisplay, unsigned short Mask)
{
register unsigned short *pDest, *d, *s;
register int iSrcPitch, iXPitch, iYPitch;
register int x, y;
register int sx, sy;

sx = ax+dx;
sy = ay+dy;

   iXPitch = gx_displayprop.cbxPitch / 2; // divide by 2 since we are working with shorts
   iYPitch = gx_displayprop.cbyPitch / 2;
   iSrcPitch = (cx + 1) & 0xfffe; // dword aligned source bitmap pitch

   pDest = (unsigned short *)pDisplay;
   pDest += (iCornerX * iXPitch + iCornerY * iYPitch);

   // Do a 16-bit pixel-based bitblt
   //First offset
   for (y=ay; y<sy; y++)
      {
      s = pSrc + y * iSrcPitch + ax;
      d = pDest + (y-ay) * iYPitch;
      for (x=ax; x<sx; x++)
         {
		  //Exception handling
		  
		   if((*s)!=(Mask))
			  *d = *s; //Here actual copying takes place
           s++;
		   d += iXPitch;
         }
      }
   
}//of GxBitBlitCut

unsigned short CalcColor(int r, int g, int b)
{

	unsigned short PixelCol;

	if (gx_displayprop.ffFormat & kfDirect565)
	{
		PixelCol = (unsigned short) (r & 0xff) << 11 | (g & 0xff) << 6 | (b & 0xff);
	}
	else
	{
		PixelCol = (unsigned short) (r & 0xff) << 10 | (g & 0xff) << 5 | (b & 0xff);
	}

	return PixelCol;
}// of CalcColor

void ClearScreen(unsigned short Color)
{
	for(long i=0;i<=(320*240-1);i++)
	{
		BackBuffer[i] = Color;
	}
	
}//of ClearScreen

void Flip(void)
{
	unsigned short * buffer;

	// Get the start of the screen memory from the GX function.
	buffer = (unsigned short *) GXBeginDraw();

	if (gx_displayprop.cbyPitch<0)
	{
		buffer = buffer + ((gx_displayprop.cyHeight-1)*(gx_displayprop.cbyPitch>>1));
	}
			
	//Kopiowanie do pamici ekranu
	memcpy(buffer, BackBuffer, 240*320*sizeof(unsigned short));
	
	GXEndDraw();


}// of Flip

void UpdateFrame(void)
{
	
	ClearScreen(CalcColor(0,0,0));

	UpdateScreen();

}

//-------------------------------------------------------------
//SimpleDX functions

//Rnd function
int Rnd(int limit)
{
	int Number;

	
	Number = rand();
	return (int)(Number / ((float)RAND_MAX / (float)(limit+1)));

}//of Rnd

//CalcPos function
//Helper function calculates new ball positions, and performs bouncing
void CalcPos(void)
{
	long i;
	DWORD gx, gy; //Grid position

	for(i=0;i<GameLevel;i++)
	{
		//Calculate new positions for balls

		//Calculate next animation frame
		(BallInfo[i].frame)++;
		if (BallInfo[i].frame>39) BallInfo[i].frame=0;

		//Translate absolute ball position to 2x2 grid position
		gx = (BallInfo[i].x)/2;
		gy = (BallInfo[i].y)/2;


		//Bouncing from lines in lines array
		//left & right
		if (BallInfo[i].vx<0 && gx <=0)
		{
			BallInfo[i].vx*= -1;
		}
		else
		if (BallInfo[i].vx<0 && lines[gx-1][gy] == 1) 
		{
			BallInfo[i].vx*= -1;
		}
		else
		if (BallInfo[i].vx>0 && gx+3 >= 119)
		{
			BallInfo[i].vx*= -1;
		}
		else
		if (BallInfo[i].vx>0 && lines[gx+4][gy] == 1) 
		{
			BallInfo[i].vx*= -1;
		}

		//up & down
		if (BallInfo[i].vy<0 && gy <= 0)
		{
			BallInfo[i].vy*= -1;
		}
		else
		if (BallInfo[i].vy<0 && lines[gx][gy-1] == 1) 
		{
			BallInfo[i].vy*= -1;
		}
		else
		if (BallInfo[i].vy>0 && gy+3 >= 159)
		{
			BallInfo[i].vy*= -1;
		}
		else
		if (BallInfo[i].vy>0 && lines[gx][gy+4] == 1) 
		{
			BallInfo[i].vy*= -1;
		}

		BallInfo[i].x+=BallInfo[i].vx;
		BallInfo[i].y+=BallInfo[i].vy;

		gx = (BallInfo[i].x)/2;
		gy = (BallInfo[i].y)/2;


		//Check if a ball hit the trail
		if (lines[gx][gy]==2  || lines[gx][gy]>=6 )
		{
			LifeOut();
			return;
		}

	}

	//Now calculate new ship position
	if (!LeaveTrail)
	{
		//Try to move into new direction
		if ( lines[sx+nvx][sy+nvy]==1 )
		{
			sx+=nvx;
			sy+=nvy;
			svx=nvx;
			svy=nvy;
		}
		else //try to move still in old direction
		if ( lines[sx+svx][sy+svy]==1 )
		{
			sx+=svx;
			sy+=svy;
		}
		
	}
	else
	if ((sx+nvx >= 4) && (sx+nvx <= 115) && (sy+nvy >= 8) && (sy+nvy <= 156))
	{
			sx+=nvx;
			sy+=nvy;
			svx=nvx;
			svy=nvy;
			if (lines[sx][sy]==1 ) 
			{
				LeaveTrail = FALSE; //We reached the line
				FillAreas(sx-nvx,sy-1,sx-svx,sy+1); //Perform filling if necessary
				FillAreas(sx-1,sy-nvy,sx+1,sy-nvy);
				BlendLines();
				DrawLines();
				DrawStatus();
			} else if (lines[sx][sy]==2) LifeOut(); //oops, hit own line...
			else if (lines[sx][sy]==0) lines[sx][sy]=2;
	}		

}//End of function CalcPos


void FillR (int iX, int iY)
{
    // Allocate the maximum amount of space needed.  If you prefer less, you
    // need to modify this data structure to allow for dynamic reallocation
    // when it is needed.  An empty stack has iTop == -1.

	int iXMax = 119;
	int iYMax = 159;
    int iQuantity = 160*120;
    int* akXStack = new int[iQuantity];
    int* akYStack = new int[iQuantity];

    // Push seed point onto stack if it has the background color.  All points
    // pushed onto stack have background color iBColor.
    int iTop = 0;
    akXStack[iTop] = iX;
    akYStack[iTop] = iY;

	if (lines[iX][iY] !=0) 
	{
		if (lines[iX][iY]==3) RNFill = 1;
		delete[] akXStack;
		delete[] akYStack;
		return;
	}

    while ( iTop >= 0 )  // stack is not empty
    {
        // Read top of stack.  Do not pop since we need to return to this
        // top value later to restart the fill in a different direction.
        iX = akXStack[iTop];
        iY = akYStack[iTop];

        // fill the pixel
        lines[iX][iY] = 4;


        int iXp1 = iX+1;
		if (lines[iXp1][iY] == 3) 
		{	
			RNFill = 1;
			delete[] akXStack;
			delete[] akYStack;
			return; //Ball encountered
		}
        if ( iXp1 < iXMax && lines[iXp1][iY] == 0 )
        {
            // push pixel with background color
            iTop++;
            akXStack[iTop] = iXp1;
            akYStack[iTop] = iY;
            continue;
        }

        int iXm1 = iX-1;
		if (lines[iXm1][iY] == 3) 
		{	
			RNFill = 1;
			delete[] akXStack;
			delete[] akYStack;
			return; //Ball encountered
		}
        if ( 0 <= iXm1 && lines[iXm1][iY] == 0 )
        {
            // push pixel with background color
            iTop++;
            akXStack[iTop] = iXm1;
            akYStack[iTop] = iY;
            continue;
        }

        int iYp1 = iY+1;
		if (lines[iX][iYp1] == 3) 
		{	
			RNFill = 1;
			delete[] akXStack;
			delete[] akYStack;
			return; //Ball encountered
		}
        if ( iYp1 < iYMax && lines[iX][iYp1] == 0 )
        {
            // push pixel with background color
            iTop++;
            akXStack[iTop] = iX;
            akYStack[iTop] = iYp1;
            continue;
        }

        int iYm1 = iY-1;
		if (lines[iX][iYm1] == 3) 
		{	
			RNFill = 1;
			delete[] akXStack;
			delete[] akYStack;
			return; //Ball encountered
		}
        if ( 0 <= iYm1 && lines[iX][iYm1] == 0 )
        {
            // push pixel with background color
            iTop++;
            akXStack[iTop] = iX;
            akYStack[iTop] = iYm1;
            continue;
        }

        // done in all directions, pop and return to search other directions
        iTop--;
    }


}
void FillL (int iX, int iY)
{
    // Allocate the maximum amount of space needed.  If you prefer less, you
    // need to modify this data structure to allow for dynamic reallocation
    // when it is needed.  An empty stack has iTop == -1.

	int iXMax = 119;
	int iYMax = 159;
    int iQuantity = 160*120;
    int* akXStack = new int[iQuantity];
    int* akYStack = new int[iQuantity];

    // Push seed point onto stack if it has the background color.  All points
    // pushed onto stack have background color iBColor.
    int iTop = 0;
    akXStack[iTop] = iX;
    akYStack[iTop] = iY;

	if (lines[iX][iY] !=0) 
	{
		if (lines[iX][iY]==3) LNFill = 1;
		delete[] akXStack;
		delete[] akYStack;
		return;
	}

    while ( iTop >= 0 )  // stack is not empty
    {
        // Read top of stack.  Do not pop since we need to return to this
        // top value later to restart the fill in a different direction.
        iX = akXStack[iTop];
        iY = akYStack[iTop];

        // fill the pixel
        lines[iX][iY] = 5;


        int iXp1 = iX+1;
		if (lines[iXp1][iY] == 3) 
		{	
			LNFill = 1;
			delete[] akXStack;
			delete[] akYStack;
			return; //Ball encountered
		}
        if ( iXp1 < iXMax && lines[iXp1][iY] == 0 )
        {
            // push pixel with background color
            iTop++;
            akXStack[iTop] = iXp1;
            akYStack[iTop] = iY;
            continue;
        }

        int iXm1 = iX-1;
		if (lines[iXm1][iY] == 3) 
		{	
			LNFill = 1;
			delete[] akXStack;
			delete[] akYStack;
			return; //Ball encountered
		}
        if ( 0 <= iXm1 && lines[iXm1][iY] == 0 )
        {
            // push pixel with background color
            iTop++;
            akXStack[iTop] = iXm1;
            akYStack[iTop] = iY;
            continue;
        }

        int iYp1 = iY+1;
		if (lines[iX][iYp1] == 3) 
		{	
			LNFill = 1;
			delete[] akXStack;
			delete[] akYStack;
			return; //Ball encountered
		}
        if ( iYp1 < iYMax && lines[iX][iYp1] == 0 )
        {
            // push pixel with background color
            iTop++;
            akXStack[iTop] = iX;
            akYStack[iTop] = iYp1;
            continue;
        }

        int iYm1 = iY-1;
		if (lines[iX][iYm1] == 3) 
		{	
			LNFill = 1;
			delete[] akXStack;
			delete[] akYStack;
			return; //Ball encountered
		}
        if ( 0 <= iYm1 && lines[iX][iYm1] == 0 )
        {
            // push pixel with background color
            iTop++;
            akXStack[iTop] = iX;
            akYStack[iTop] = iYm1;
            continue;
        }

        // done in all directions, pop and return to search other directions
        iTop--;
    }


}
void FillAreas(int ax, int ay,int bx, int by)
{
	int gx,gy, cl;
	int i, l;
	//Mark ball positions into lines grid
	for(i=0;i<GameLevel;i++)
	{
		gx = (BallInfo[i].x)/2;
		gy = (BallInfo[i].y)/2;
		lines[gx][gy]=3;
	}

	//Now perform filling
	RNFill=0;
	LNFill=0;
	FillR(ax,ay);
	FillL(bx,by);

	cl=Rnd(7)+6;
	//cl=10;

	//Blend 
	for (i=0;i<=119;i++)
		for(l=0;l<=159;l++)
		{
			if (lines[i][l]==4) 
			{
				if (RNFill==0) lines[i][l]=cl; else lines[i][l]=0;
			}
			else
			if (lines[i][l]==5) 
			{
				if (LNFill==0) lines[i][l]=cl; else lines[i][l]=0;
			}else
			if (lines[i][l]==3) lines[i][l]=0; 
			
			/*else
			{
				//if (lines[i][l]==0)
				//lines[i][l]=200; 
			}*/
		}


		//Compute fill status
		Fill=0;
		for (i=0;i<=119;i++)
		for(l=0;l<=159;l++)
		{
			if (lines[i][l]!=0) Fill++;
		}

		Fill=Fill-960;
		if (Fill>12000) NextLevel(); //~70%
		

}// End of function FillAreas

//BlendLines function
//Changes trail lines into normal lines
void BlendLines(void)
{
	int x, y;

	for( x=0; x<=119; x++)
		for( y=0; y<=159; y++)
			if (lines[x][y]==2) lines[x][y]=1;

}//End of function BlendLines

void ResetLevel(void)
{
	sx=4;
    sy=80;
	svx = 0; //This applies to displacement
	svy = 0;
	nvx = 0; //New direction (ordered direction)
	nvy = 0;
	SIcon = 1;

	LeaveTrail=FALSE; //Are we painting new trail?
	//Initialize lines array (frame around with space on top for score line)
	LinesInit();

	Fill=0;
	LevelTime=90;
	ClearPlayField();
	DrawLines();
	DrawStatus();
}

void LifeOut(void)
{
		//MessageBox(g_hWnd, "", "LifeOUT", MB_OK);
InGame = FALSE;
Ships=Ships-1;
if (Ships<1) 
{
	GameOver();//GameOver
	return;
}
else 
{
	//DrawSplash();
	DrawSplash(2);
}

}//of LifeOut

//NextLevel function
void NextLevel(void)
{
	//Clear lines field

BallInit();
GameLevel++;
LinesInit();
	sx=4;
    sy=80;
	svx = 0; //This applies to displacement
svy = 0;
nvx = 0; //New direction (ordered direction)
nvy = 0;
SIcon = 1;

LeaveTrail=FALSE; //Are we painting new trail?


Fill=0;
LevelTime=90;
ClearPlayField();
/*DrawLines();
DrawStatus();*/
InGame=FALSE;
DrawSplash(3);

}//of NextLevel 

void GameOver(void)
{
	//MessageBox(g_hWnd, "GameOVER", "Game OVER - jumpin to begining", MB_OK);
	Ships=3;
	GameLevel=0;
	InGame=FALSE;
	IsGameOver=TRUE;
	DrawSplash(4);
//	g_bSplash=TRUE;

}




//------------------------------------------------------------------
//DirectX dependent functions
//DrawLines function
//Draws the lines from the lines array onto PlayField
BOOL DrawLines(void)
{
	
	int i,l;

	for(i=0;i<=119;i++)
		for(l=0;l<=159;l++)
		{
			//Blit the next 2x2 square
			if ((lines[i][l] == 1) || (lines[i][l] == 2))
			{
//				rcRect.left = 70+(lines[i][l]-1)*2;
//				rcRect.top = 16;
				
		     GxBitbltCut(pFramesBitmap, i*2, l*2,  70+(lines[i][l]-1)*2,16, 2,2, FramesWidth, FramesHeight, memstartPlayField, CalcColor(0,0,0));
				
				
			}//of if


			//Blit the next 2x2 square of filled area
			else if ((lines[i][l] >= 6))
			{		     
			   GxBitbltCut(pFramesBitmap, i*2, l*2,  72+(lines[i][l]-5)*2,16, 2,2, FramesWidth, FramesHeight, memstartPlayField, CalcColor(0,0,0));
			}//of if
	
		}

		return TRUE;
}// End of function DrawLines


//Draw number on PlayField, only 2 digit numbers are supported
void OutputDigits(int Number, int X, int Y)
{
	int tens;
	int units;
	//Find out the number for tens
	tens = Number / 10;
	units = Number % 10;

	//Draw first digit
	//Digits are taken from Frames bitmap, the start of digits is at 70,18
	//end every digit is 8 pixels wide and 9 pixels high
	GxBitbltCut(pFramesBitmap, X, Y,  70+(tens*8), 18, 8, 9, FramesWidth, FramesHeight, memstartPlayField, CalcColor(255,255,255));
	GxBitbltCut(pFramesBitmap, X+8, Y,  70+(units*8), 18, 8, 9, FramesWidth, FramesHeight, memstartPlayField, CalcColor(255,255,255));
}

//DrawStatus function
//Draw status line above play area
void DrawStatus(void)
{
  //Blit status background
  GxBitbltCut(pStatusBitmap, 0, 0,  0, 0, 240, 14, StatusWidth, StatusHeight, memstartPlayField, CalcColor(0,0,0));

  //Now draw Time digits
  OutputDigits(LevelTime, 43, 4);

  //Draw Fill digits
 OutputDigits((Fill*100/15914), 102, 4);

  //Draw Ships digits
 OutputDigits(Ships, 165, 4);


 //Finally draw Level digits
OutputDigits(GameLevel, 222, 4);


}//of DrawStatus

//Draws only Time digits
void DrawTime(void)
{
	//Now draw Time digits
  OutputDigits(LevelTime, 43, 4);

}//of DrawTime

//ClearPlayField
void ClearPlayField(void)
{
	for(long i=0;i<=(320*240-1);i++)
	{
		PlayField[i] = 0;
	}

}//of ClearPlayField

//UpdateScreen function
//Calls DrawBalls function and than performs surface flipping
void UpdateScreen(void)
{
	//HRESULT		hRes;
/*	DWORD		CurrentTime;

	//Slow down gameplay
	CurrentTime=GetTickCount();
	if ((CurrentTime - LastTime) >= GameDelay)
	{
		LastTime = CurrentTime;

	}
	else return;

	//Read keyboard input
	ReadInput();

	if (!g_bSplash){
	if (DrawBalls(g_hWnd) != TRUE) return;
	CalcPos();} */

	//Read keyboard input
	//ReadInput();

	DrawBalls();
	CalcPos();

	Flip();
}// End of function UpdateScreen


void DrawSplash(int Type)
{

	ClearScreen(CalcColor(0,0,0));

	switch (Type)
	{
	//intro
	case 1:
	GxBitbltCut(pSplashBitmap, 20, 90,  0, 0, 200, 100, SplashWidth, SplashHeight, memstartBackBuffer, CalcColor(255,255,255));
	break;
	//ship lost
	case 2:
	GxBitbltCut(pSplashBitmap, 20, 140,  0, 100, 200, 20, SplashWidth, SplashHeight, memstartBackBuffer, CalcColor(255,255,255));
	break;
	//next level
	case 3:
	GxBitbltCut(pSplashBitmap, 20, 130,  0, 120, 200, 30, SplashWidth, SplashHeight, memstartBackBuffer, CalcColor(255,255,255));
	break;
	//game over
	case 4:
	GxBitbltCut(pSplashBitmap, 20, 120,  0, 150, 200, 50, SplashWidth, SplashHeight, memstartBackBuffer, CalcColor(255,255,255));
	break;
	}
	Flip();
	
}// of DrawSplash

//DrawBalls function
//Clears back surface and draw new balls on it
BOOL DrawBalls(void)
{
//	RECT			rcRect; //Rectangle for blitter operations
	//HRESULT			hRes;
//	DDBLTFX			BltParams;


	//If ship is leaving the trail paint it
	
	//Blit the next 4x4 square
	
	if (LeaveTrail==TRUE)
	{
	    GxBitbltCut(pFramesBitmap, sx*2, sy*2, 72,16,2,2, FramesWidth, FramesHeight, memstartPlayField, CalcColor(0,0,0));
	}

//PlayField->BackBuffer
	memcpy(BackBuffer, PlayField, 320*240*sizeof(unsigned short));


		
	//Draw members of BallInfo array
	for(long i=0;i<GameLevel;i++)
	{		
		GxBitbltCut(pFramesBitmap, BallInfo[i].x, BallInfo[i].y, BallInfo[i].frame * 8,BallInfo[i].type  * 8,8,8, FramesWidth, FramesHeight, memstartBackBuffer, CalcColor(0,0,0));
		
	}// of for

	//Draw ship

	GxBitbltCut(pFramesBitmap, (sx*2-7), (sy*2-7), SIcon * 16, 16,16,16, FramesWidth, FramesHeight, memstartBackBuffer, CalcColor(0,0,0));

	return TRUE;
	

}// End of function DrawBalls

//ReadInput function
//Reads user input and sets global variables accordingly
BOOL ReadInput(WPARAM wParam)
{

	if (wParam == (unsigned) gx_keylist.vkStart)
	{
		return FALSE;
	};

	if (wParam == (unsigned) gx_keylist.vkRight)
	{
		if (lines[sx+nvx][sy+nvy]==0) LeaveTrail = TRUE;
		nvx = 1;
		nvy = 0;
		SIcon = 1;
		return TRUE;
	};

		if (wParam == (unsigned) gx_keylist.vkLeft)
	{
		if (lines[sx+nvx][sy+nvy]==0) LeaveTrail = TRUE;
		nvx = -1;
		nvy = 0;
		SIcon = 2;
		return TRUE;
	};

		if (wParam == (unsigned) gx_keylist.vkUp)
	{
		if (lines[sx+nvx][sy+nvy]==0) LeaveTrail = TRUE;
		nvx = 0;
		nvy = -1;
		SIcon = 0;
		return TRUE;
	};

		if (wParam == (unsigned) gx_keylist.vkDown)
	{
		if (lines[sx+nvx][sy+nvy]==0) LeaveTrail = TRUE;
		nvx = 0;
		nvy = 1;
		SIcon = 3;
		return TRUE;
	};	

	return TRUE;

}//of ReadInput

void BallInit()
{
	srand( (int)GetTickCount());
	int prob[10]={1,1,-1,-1,2,1,-1,-2,1,-1};

	for (long i=0;i<=99;i++)
	{
		BallInfo[i].x = Rnd(180)+5;
		BallInfo[i].y = Rnd(280)+15;
		
		BallInfo[i].vx = prob[Rnd(10)];
		BallInfo[i].vy = prob[Rnd(10)];
		//while ((BallInfo[i].vy = Rnd(1)-1)==0);
		
		//BallInfo[i].vx = BallInfo[i].vx;
		//BallInfo[i].vy = BallInfo[i].vy;

		BallInfo[i].type = Rnd(1);
		if (BallInfo[i].type > 1) BallInfo[i].type = 1;
		BallInfo[i].frame = Rnd(39);
		if (BallInfo[i].frame > 39) BallInfo[i].frame = 39;
	}

}// of BallInit

void LinesInit()
{
	
	memset(lines, 0, sizeof(int)*(160*120));

	//Note the border for ship sprite
	for (int i=8;i<=156;i++) 
	{
		lines[3][i]=100;//Safety frame to prevent leving trail outside
		lines[4][i]=1; 
		lines[115][i]=1;
		lines[116][i]=100;
	}
	
	for (i=4;i<=115;i++) 
	{
		lines[i][7]=100;
		lines[i][8]=1;//32 pixel status line
		lines[i][156]=1;
		lines[i][157]=100;
	}

	//Note that the play field is therefore (114-5)x(155-9) squares = 15914
	//This is important for Fill % computations 

}//of LinesInit